# tests/test_core.py

import pytest
from grapes_tokenizer.core import GrapesTokenizer

@ pytest.mark.parametrize("text, expected", [
    # Letters with positional weighting: value * (position index + 1)
    ("a", 1 * 1),                              # 'a'=1 at pos0
    ("abc", 1*1 + 2*2 + 3*3),                 # 1 + 4 + 9 = 14
    ("xyz", 24*1 + 25*2 + 26*3),              # 24 + 50 + 78 = 152
    ("Hello", 8*1 + 5*2 + 12*3 + 12*4 + 15*5), # weighted sum
])
def test_simple_letters(text, expected):
    tok = GrapesTokenizer()
    assert tok.encode(text) == expected


def test_digits_mapping():
    tok = GrapesTokenizer()
    # '0'->0*1=0, '5'->5*2=10, '9'->9*3=27 => 37
    assert tok.encode("059") == 0 + 10 + 27


def test_specials_mapping():
    tok = GrapesTokenizer()
    # space=32*1=32, '!'=33*2=66, '@'=64*3=192 => 290
    assert tok.encode(" !@") == 32 + 66 + 192


def test_mixed_characters():
    tok = GrapesTokenizer()
    # "a1!b": a=1*1=1, '1'=1*2=2, '!'=33*3=99, b=2*4=8 => 110
    assert tok.encode("a1!b") == 1 + 2 + 99 + 8


def test_empty_string():
    tok = GrapesTokenizer()
    assert tok.encode("") == 0


def test_long_text_performance():
    # a mix of printable ASCII characters repeated
    long_text = "".join(chr((i % 94) + 33) for i in range(1000))
    tok = GrapesTokenizer()
    result = tok.encode(long_text)
    assert isinstance(result, int)
    assert result > 0



# grapes_tokenizer/core.py

class GrapesTokenizer:
    """
    Sum-based tokenizer with:
      - Letters a–z → 1–26
      - Digits 0–9 → their integer value (0–9)
      - All other characters → ASCII code via ord()
    Always uses positional weighting.
    """

    def __init__(self, case_sensitive: bool = False):
        self.case_sensitive = case_sensitive
        self.positional = True

    def _char_to_value(self, ch: str) -> int:
        if not self.case_sensitive:
            ch = ch.lower()
        if ch.isalpha():
            return ord(ch) - ord('a') + 1
        if ch.isdigit():
            return int(ch)
        return ord(ch)

    def _binary_add(self, b1: str, b2: str) -> str:
        return bin(int(b1, 2) + int(b2, 2))[2:]

    def encode(self, text: str) -> int:
        """
        Encode a string to its positional sum-based token (decimal).
        """
        total_bin = '0'
        for idx, ch in enumerate(text):
            val = self._char_to_value(ch)
            # apply positional weighting
            val *= (idx + 1)
            b = bin(val)[2:]
            total_bin = self._binary_add(total_bin, b)
        return int(total_bin, 2)



# setup.py
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="grapes-tokenizer",
    version="0.1.3",
    author="Akash Pittalwar",
    author_email="akashpittalwar107@gmail.com",
    description="A simple sum-based tokenizer for NLP/LLM research",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/akashpittalwar/Grapes-tokenizer",
    project_urls={
        "Source": "https://github.com/akashpittalwar/Grapes-tokenizer",
        "Tracker": "https://github.com/akashpittalwar/Grapes-tokenizer/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(exclude=["tests*", "examples*"]),
    python_requires=">=3.6",
)
